"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../../src");
const chai = require("chai");
const expect = chai.expect;
chai.use(require("chai-as-promised"));
const BtcMccConnection = {
    url: process.env.BTC_URL || "",
    username: process.env.BTC_USERNAME || "",
    password: process.env.BTC_PASSWORD || "",
};
describe("Transaction Btc base test ", function () {
    let MccClient;
    before(async function () {
        src_1.traceManager.displayStateOnException = false;
        MccClient = new src_1.MCC.BTC(BtcMccConnection);
    });
    describe("Transaction does not exist ", function () {
        const txid = "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d37786156ff";
        it("Should get Invalid Transaction error ", async function () {
            await expect(MccClient.getTransaction(txid)).to.be.rejectedWith("InvalidTransaction");
        });
    });
    describe("Transaction full ", function () {
        const txid = "141479db9d6da30fafaf47e71ae6323cac57c391ea2f7f84754e0a70fea8e36a";
        let transaction;
        before(async () => {
            let fullTrans = await MccClient.getTransaction(txid);
            if (fullTrans) {
                transaction = new src_1.BtcTransaction(fullTrans.data);
                await transaction.makeFullPayment(MccClient);
                // console.log(mccJsonStringify(transaction.additionalData));
            }
        });
        it("Should find transaction in block ", function () {
            expect(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", async function () {
            expect(transaction.txid).to.eq(txid);
        });
        it("Should get standardized txid ", async function () {
            expect(transaction.stdTxid).to.eq(txid);
        });
        it("Should get transaction hash ", async function () {
            expect(transaction.hash).to.eq("f3180575f7662b49ee6267c424d894b84f7d0ef72405c02b18677a683d4052af");
        });
        it("Should get transaction reference array ", async function () {
            expect(transaction.reference.length).to.eq(0);
        });
        it("Should get standardized transaction reference ", async function () {
            expect(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
        });
        it("Should get transaction timestamp ", async function () {
            expect(transaction.unixTimestamp).to.eq(1647613710);
        });
        it("Should get source address ", async function () {
            expect(transaction.sourceAddresses.length).to.eq(3);
            expect(transaction.sourceAddresses[0]).to.eq("bc1q38lr3a45xtlz8032sz8xwc72gs652wfcq046pzxtxx6c70nvpessnc8dyk");
        });
        it("Should get receiving address ", async function () {
            expect(transaction.receivingAddresses.length).to.eq(6);
            expect(transaction.receivingAddresses[0]).to.eq("19Vxz7mg6YSgQhVB96YvXFsES1e8aXewHp");
        });
        it("Should get fee ", async function () {
            expect(transaction.fee.toNumber()).to.eq(43300);
        });
        it("Should spend amount ", async function () {
            expect(transaction.spentAmounts.length).to.eq(3);
            expect(transaction.spentAmounts[0].address).to.eq("bc1q38lr3a45xtlz8032sz8xwc72gs652wfcq046pzxtxx6c70nvpessnc8dyk");
            expect(transaction.spentAmounts[0].amount.toNumber()).to.eq(7424891554);
        });
        it("Should received amount ", async function () {
            expect(transaction.receivedAmounts.length).to.eq(6);
            expect(transaction.receivedAmounts[0].address).to.eq("19Vxz7mg6YSgQhVB96YvXFsES1e8aXewHp");
            expect(transaction.receivedAmounts[0].amount.toNumber()).to.eq(237272503);
        });
        it("Should get type ", async function () {
            expect(transaction.type).to.eq("full_payment");
        });
        it("Should check if native payment ", async function () {
            expect(transaction.isNativePayment).to.eq(true);
        });
        it("Should get currency name ", async function () {
            expect(transaction.currencyName).to.eq("BTC");
        });
        it("Should get elementary unit ", async function () {
            expect(transaction.elementaryUnits.toNumber()).to.eq(100000000);
        });
        it("Should get success status ", async function () {
            expect(transaction.successStatus).to.eq(0);
        });
    });
    const TransactionsToTest = [
        {
            description: "Transaction with one reference",
            txid: "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d3778615684",
            expect: {
                txid: "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d3778615684",
                stdTxid: "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d3778615684",
                hash: "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d3778615684",
                reference: ["636861726c6579206c6f766573206865696469"],
                stdPaymentReference: "0x0000000000000000000000000000000000000000000000000000000000000000",
                unixTimestamp: 1404107109,
                sourceAddresses: [undefined],
                receivingAddresses: ["1HnhWpkMHMjgt167kvgcPyurMmsCQ2WPgg", undefined],
                isFeeError: true,
                fee: "OutsideError",
                spentAmounts: [
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                ],
                receivedAmounts: [
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                    {
                        address: "1HnhWpkMHMjgt167kvgcPyurMmsCQ2WPgg",
                        amount: (0, src_1.toBN)(200000),
                    },
                ],
                type: "payment",
                isNativePayment: true,
                currencyName: "BTC",
                elementaryUnits: "100000000",
                successStatus: src_1.TransactionSuccessStatus.SUCCESS,
            },
            summary: {
                isNativePayment: true,
                sourceAddress: "1HnhWpkMHMjgt167kvgcPyurMmsCQ2WPgg",
                receivingAddress: undefined,
                spentAmount: (0, src_1.toBN)(20000),
                receivedAmount: (0, src_1.toBN)(0),
                paymentReference: "0x0000000000000000000000000000000000000000000000000000000000000000",
                oneToOne: false,
                isFull: true,
            },
        },
        {
            description: "Transaction with multiple vins",
            txid: "16920c5619b4c43fd5c9c0fc594153f2bf1a80c930238a8ee870aece0bc7cc59",
            expect: {
                txid: "16920c5619b4c43fd5c9c0fc594153f2bf1a80c930238a8ee870aece0bc7cc59",
                stdTxid: "16920c5619b4c43fd5c9c0fc594153f2bf1a80c930238a8ee870aece0bc7cc59",
                hash: "dda6777e407602bcec51963278b071a95a3a3f382fb73429ab203769d658da08",
                reference: [],
                stdPaymentReference: "0x0000000000000000000000000000000000000000000000000000000000000000",
                unixTimestamp: 1647547988,
                sourceAddresses: [undefined, undefined, undefined],
                receivingAddresses: ["bc1q7ydxwryw7u6xkkzhlddugv8hyzsd6u6c8zr7rc", "14PbdXD3gRMnrrsP4CnS66fYKHSb1aawea"],
                isFeeError: true,
                fee: "OutsideError",
                spentAmounts: [
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                ],
                receivedAmounts: [
                    {
                        address: "bc1q7ydxwryw7u6xkkzhlddugv8hyzsd6u6c8zr7rc",
                        amount: (0, src_1.toBN)(2259),
                    },
                    {
                        address: "14PbdXD3gRMnrrsP4CnS66fYKHSb1aawea",
                        amount: (0, src_1.toBN)(12240),
                    },
                ],
                type: "payment",
                isNativePayment: true,
                currencyName: "BTC",
                elementaryUnits: "100000000",
                successStatus: src_1.TransactionSuccessStatus.SUCCESS,
            },
            summary: {
                isNativePayment: true,
                sourceAddress: 'bc1qtwha4x2kcm6z05z4hn88atye3wq7aatrljrjly',
                receivingAddress: 'bc1q7ydxwryw7u6xkkzhlddugv8hyzsd6u6c8zr7rc',
                spentAmount: (0, src_1.toBN)(3533),
                receivedAmount: (0, src_1.toBN)(-4405),
                paymentReference: '0x0000000000000000000000000000000000000000000000000000000000000000',
                oneToOne: false,
                isFull: true
            }
        },
        {
            description: "Coinbase transaction with more reference",
            txid: "fbf351cd9f1a561be21e3977282e931c5209ed6b90472e225b4a674dbc643511",
            expect: {
                txid: "fbf351cd9f1a561be21e3977282e931c5209ed6b90472e225b4a674dbc643511",
                stdTxid: "fbf351cd9f1a561be21e3977282e931c5209ed6b90472e225b4a674dbc643511",
                hash: "0d7d19de1dff1bb5873d1cb790cb4087d5e3ba7607e001d4d5870b1e87872dc4",
                reference: [
                    "aa21a9ed7e2e11c040dbddce689e9a075f91f0bfa408cf3dadc229a0993ded3b4a9423e2",
                    "b9e11b6deaba71a095ba7fa47426cabe0b340a393b6eeecce598af3020a413ec1d7116e7",
                    "52534b424c4f434b3ab51f35a72153280cffb0721d9196f11c4830ebf63866e84ce24fae2c003e4e48",
                ],
                stdPaymentReference: "0x0000000000000000000000000000000000000000000000000000000000000000",
                unixTimestamp: 1644814708,
                sourceAddresses: [undefined],
                receivingAddresses: ["1JvXhnHCi6XqcanvrZJ5s2Qiv4tsmm2UMy", undefined, undefined, undefined],
                isFeeError: false,
                fee: "0",
                spentAmounts: [{ address: undefined, amount: (0, src_1.toBN)(0) }],
                receivedAmounts: [
                    {
                        address: "1JvXhnHCi6XqcanvrZJ5s2Qiv4tsmm2UMy",
                        amount: (0, src_1.toBN)(632706642),
                    },
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                ],
                type: "coinbase",
                isNativePayment: true,
                currencyName: "BTC",
                elementaryUnits: "100000000",
                successStatus: src_1.TransactionSuccessStatus.SUCCESS,
            },
            summary: {
                isNativePayment: true,
                sourceAddress: undefined,
                receivingAddress: '1JvXhnHCi6XqcanvrZJ5s2Qiv4tsmm2UMy',
                spentAmount: (0, src_1.toBN)(0),
                receivedAmount: (0, src_1.toBN)(632706642),
                paymentReference: '0x0000000000000000000000000000000000000000000000000000000000000000',
                oneToOne: false,
                isFull: false
            }
        },
    ];
    for (let transData of TransactionsToTest) {
        describe(transData.description, function () {
            let transaction;
            before(async function () {
                let transactionb = await MccClient.getTransaction(transData.txid);
                if (transactionb !== null) {
                    transaction = transactionb;
                }
            });
            it("Should find transaction in block ", function () {
                expect(transaction).to.not.eq(undefined);
            });
            it("Should get transaction txid ", async function () {
                expect(transaction.txid).to.eq(transData.expect.txid);
            });
            it("Should get standardized txid ", async function () {
                expect(transaction.stdTxid).to.eq(transData.expect.stdTxid);
            });
            it("Should get transaction hash ", async function () {
                expect(transaction.hash).to.eq(transData.expect.hash);
            });
            it("Should get transaction reference array ", async function () {
                expect(transaction.reference.length).to.eq(transData.expect.reference.length);
                const a = transaction.reference.sort();
                const b = transData.expect.reference.sort();
                for (let i = 0; i < a.length; i++) {
                    expect(a[i]).to.eq(b[i]);
                }
            });
            it("Should get standardized transaction reference ", async function () {
                expect(transaction.stdPaymentReference).to.eq(transData.expect.stdPaymentReference);
            });
            it("Should get transaction timestamp ", async function () {
                expect(transaction.unixTimestamp).to.eq(transData.expect.unixTimestamp);
            });
            it("Should get source address ", async function () {
                expect(transaction.sourceAddresses.length).to.eq(transData.expect.sourceAddresses.length);
                const a = transaction.sourceAddresses.sort();
                const b = transData.expect.sourceAddresses.sort();
                for (let i = 0; i < a.length; i++) {
                    expect(a[i]).to.eq(b[i]);
                }
            });
            it("Should get receiving address ", async function () {
                expect(transaction.receivingAddresses.length).to.eq(transData.expect.receivingAddresses.length);
                const a = transaction.receivingAddresses.sort();
                const b = transData.expect.receivingAddresses.sort();
                for (let i = 0; i < a.length; i++) {
                    expect(a[i]).to.eq(b[i]);
                }
            });
            it("Should get fee ", async function () {
                if (transData.expect.isFeeError) {
                    expect(function () {
                        transaction.fee;
                    }).to.throw(transData.expect.fee);
                }
                else {
                    expect(transaction.fee.toString()).to.eq(transData.expect.fee);
                }
            });
            it("Should spend amount ", async function () {
                expect(transaction.spentAmounts.length).to.eq(transData.expect.spentAmounts.length);
                const a = transaction.spentAmounts.sort();
                const b = transData.expect.spentAmounts.sort();
                for (let i = 0; i < a.length; i++) {
                    expect(a[i].address).to.eq(b[i].address);
                    expect(a[i].amount.toString()).to.eq(b[i].amount.toString());
                }
            });
            it("Should received amount ", async function () {
                expect(transaction.receivedAmounts.length).to.eq(transData.expect.receivedAmounts.length);
                const a = transaction.receivedAmounts.sort();
                const b = transData.expect.receivedAmounts.sort();
                for (let i = 0; i < a.length; i++) {
                    expect(a[i].address).to.eq(b[i].address);
                    expect(a[i].amount.toString()).to.eq(b[i].amount.toString());
                }
            });
            it("Should get type ", async function () {
                expect(transaction.type).to.eq(transData.expect.type);
            });
            it("Should check if native payment ", async function () {
                expect(transaction.isNativePayment).to.eq(transData.expect.isNativePayment);
            });
            it("Should get currency name ", async function () {
                expect(transaction.currencyName).to.eq(transData.expect.currencyName);
            });
            it("Should get elementary unit ", async function () {
                expect(transaction.elementaryUnits.toString()).to.eq(transData.expect.elementaryUnits);
            });
            it("Should get success status ", async function () {
                expect(transaction.successStatus).to.eq(transData.expect.successStatus);
            });
            it("Should get payment summary", async function () {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
                const summary = await transaction.paymentSummary(MccClient, 0, 0, true);
                if (transData.expect.isNativePayment) {
                    expect(summary.isNativePayment).to.eq((_a = transData.summary) === null || _a === void 0 ? void 0 : _a.isNativePayment);
                    expect(summary.sourceAddress).to.eq((_b = transData.summary) === null || _b === void 0 ? void 0 : _b.sourceAddress);
                    expect(summary.receivingAddress).to.eq((_c = transData.summary) === null || _c === void 0 ? void 0 : _c.receivingAddress);
                    expect((_d = summary.spentAmount) === null || _d === void 0 ? void 0 : _d.toString()).to.eq((_f = (_e = transData.summary) === null || _e === void 0 ? void 0 : _e.spentAmount) === null || _f === void 0 ? void 0 : _f.toString());
                    expect((_g = summary.receivedAmount) === null || _g === void 0 ? void 0 : _g.toString()).to.eq((_j = (_h = transData.summary) === null || _h === void 0 ? void 0 : _h.receivedAmount) === null || _j === void 0 ? void 0 : _j.toString());
                    expect(summary.paymentReference).to.eq((_k = transData.summary) === null || _k === void 0 ? void 0 : _k.paymentReference);
                    expect(summary.oneToOne).to.eq((_l = transData.summary) === null || _l === void 0 ? void 0 : _l.oneToOne);
                    expect(summary.isFull).to.eq((_m = transData.summary) === null || _m === void 0 ? void 0 : _m.isFull);
                }
                else {
                    expect(summary.isNativePayment).to.eq((_o = transData.summary) === null || _o === void 0 ? void 0 : _o.isNativePayment);
                }
            });
        });
    }
});
// TODO special transactions
// https://www.blockchain.com/btc/tx/56214420a7c4dcc4832944298d169a75e93acf9721f00656b2ee0e4d194f9970
// https://www.blockchain.com/btc/tx/055f9c6dc094cf21fa224e1eb4a54ee3cc44ae9daa8aa47f98df5c73c48997f9
//# sourceMappingURL=transaction.test.js.map